pub struct Preceded<Fst, Snd>(pub Fst, pub Snd);Expand description
Combinator that sequentially applies two combinators and returns the result of the second one.
Tuple Fields
0: Fst1: SndTrait Implementations
Sourceimpl<'x, I, O, Fst, Snd> Combinator<'x, I, O> for Preceded<Fst, Snd>where
I: VestInput,
O: VestOutput<I>,
Fst: Combinator<'x, I, O, Type = (), SType = ()>,
Snd: Combinator<'x, I, O>,
Fst::V: SecureSpecCombinator<Type = ()>,
Snd::V: SecureSpecCombinator<Type = <Snd::Type as View>::V>,
impl<'x, I, O, Fst, Snd> Combinator<'x, I, O> for Preceded<Fst, Snd>where
I: VestInput,
O: VestOutput<I>,
Fst: Combinator<'x, I, O, Type = (), SType = ()>,
Snd: Combinator<'x, I, O>,
Fst::V: SecureSpecCombinator<Type = ()>,
Snd::V: SecureSpecCombinator<Type = <Snd::Type as View>::V>,
Sourceopen spec fn ex_requires(&self) -> bool
open spec fn ex_requires(&self) -> bool
{ (&self.0, &self.1).ex_requires() }Sourceexec fn serialize<'b>(
&self,
v: Self::SType,
data: &'b mut O,
pos: usize,
) -> Result<usize, SerializeError>
exec fn serialize<'b>( &self, v: Self::SType, data: &'b mut O, pos: usize, ) -> Result<usize, SerializeError>
Sourcetype Type = <Snd as Combinator<'x, I, O>>::Type
type Type = <Snd as Combinator<'x, I, O>>::Type
The result type of parsing
Sourcetype SType = <Snd as Combinator<'x, I, O>>::SType
type SType = <Snd as Combinator<'x, I, O>>::SType
The input type of serialization, often a reference to
Self::Type.
For “structural” formats though (e.g., crate::regular::sequence::Pair and crate::regular::variant::Choice),
this is the tuple/sum of the corresponding Combinator::SType types.Sourceimpl<U1, U2, V1, V2> DisjointFrom<Preceded<U2, V2>> for Preceded<U1, V1>where
U1: DisjointFrom<U2> + SecureSpecCombinator<Type = ()>,
U2: SecureSpecCombinator<Type = ()>,
V1: SpecCombinator,
V2: SpecCombinator,
impl<U1, U2, V1, V2> DisjointFrom<Preceded<U2, V2>> for Preceded<U1, V1>where
U1: DisjointFrom<U2> + SecureSpecCombinator<Type = ()>,
U2: SecureSpecCombinator<Type = ()>,
V1: SpecCombinator,
V2: SpecCombinator,
Sourceopen spec fn disjoint_from(&self, other: &Preceded<U2, V2>) -> bool
open spec fn disjoint_from(&self, other: &Preceded<U2, V2>) -> bool
{ self.0.disjoint_from(&other.0) }Sourceproof fn parse_disjoint_on(&self, other: &Preceded<U2, V2>, buf: Seq<u8>)
proof fn parse_disjoint_on(&self, other: &Preceded<U2, V2>, buf: Seq<u8>)
Sourceimpl<Fst: SecureSpecCombinator<Type = ()>, Snd: SecureSpecCombinator> SecureSpecCombinator for Preceded<Fst, Snd>
impl<Fst: SecureSpecCombinator<Type = ()>, Snd: SecureSpecCombinator> SecureSpecCombinator for Preceded<Fst, Snd>
Sourceproof fn theorem_serialize_parse_roundtrip(&self, v: Self::Type)
proof fn theorem_serialize_parse_roundtrip(&self, v: Self::Type)
Sourceproof fn theorem_parse_serialize_roundtrip(&self, buf: Seq<u8>)
proof fn theorem_parse_serialize_roundtrip(&self, buf: Seq<u8>)
Sourceopen spec fn is_prefix_secure() -> bool
open spec fn is_prefix_secure() -> bool
{ Fst::is_prefix_secure() && Snd::is_prefix_secure() }Sourceproof fn lemma_prefix_secure(&self, s1: Seq<u8>, s2: Seq<u8>)
proof fn lemma_prefix_secure(&self, s1: Seq<u8>, s2: Seq<u8>)
Sourceproof fn lemma_parse_length(&self, s: Seq<u8>)
proof fn lemma_parse_length(&self, s: Seq<u8>)
Sourceopen spec fn is_productive(&self) -> bool
open spec fn is_productive(&self) -> bool
{ (self.0, self.1).is_productive() }Sourceproof fn lemma_parse_productive(&self, s: Seq<u8>)
proof fn lemma_parse_productive(&self, s: Seq<u8>)
Sourcefn corollary_parse_surjective(&self, v: Self::Type)
fn corollary_parse_surjective(&self, v: Self::Type)
Sourcefn corollary_serialize_injective_contraposition(
&self,
v1: Self::Type,
v2: Self::Type,
)
fn corollary_serialize_injective_contraposition( &self, v1: Self::Type, v2: Self::Type, )
Sourcefn lemma_serialize_productive(&self, v: Self::Type)
fn lemma_serialize_productive(&self, v: Self::Type)
Sourceimpl<Fst: SecureSpecCombinator<Type = ()>, Snd: SpecCombinator> SpecCombinator for Preceded<Fst, Snd>
impl<Fst: SecureSpecCombinator<Type = ()>, Snd: SpecCombinator> SpecCombinator for Preceded<Fst, Snd>
Sourceopen spec fn spec_parse(&self, s: Seq<u8>) -> Option<(int, Self::Type)>
open spec fn spec_parse(&self, s: Seq<u8>) -> Option<(int, Self::Type)>
{
if let Some((n, ((), v))) = (self.0, self.1).spec_parse(s) {
Some((n, v))
} else {
None
}
}Sourceopen spec fn spec_serialize(&self, v: Self::Type) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::Type) -> Seq<u8>
{ (self.0, self.1).spec_serialize(((), v)) }Sourcetype Type = <Snd as SpecCombinator>::Type
type Type = <Snd as SpecCombinator>::Type
The view of [
Combinator::Result].Auto Trait Implementations
impl<Fst, Snd> Freeze for Preceded<Fst, Snd>
impl<Fst, Snd> RefUnwindSafe for Preceded<Fst, Snd>where
Fst: RefUnwindSafe,
Snd: RefUnwindSafe,
impl<Fst, Snd> Send for Preceded<Fst, Snd>
impl<Fst, Snd> Sync for Preceded<Fst, Snd>
impl<Fst, Snd> Unpin for Preceded<Fst, Snd>
impl<Fst, Snd> UnwindSafe for Preceded<Fst, Snd>where
Fst: UnwindSafe,
Snd: UnwindSafe,