pub struct Terminated<Fst, Snd>(pub Fst, pub Snd);Expand description
Combinator that sequentially applies two combinators and returns the result of the first one.
Tuple Fields
0: Fst1: SndTrait Implementations
Sourceimpl<'x, I, O, Fst, Snd> Combinator<'x, I, O> for Terminated<Fst, Snd>where
I: VestInput,
O: VestOutput<I>,
Fst: Combinator<'x, I, O>,
Snd: Combinator<'x, I, O, Type = (), SType = ()>,
Fst::V: SecureSpecCombinator<Type = <Fst::Type as View>::V>,
Snd::V: SecureSpecCombinator<Type = ()>,
impl<'x, I, O, Fst, Snd> Combinator<'x, I, O> for Terminated<Fst, Snd>where
I: VestInput,
O: VestOutput<I>,
Fst: Combinator<'x, I, O>,
Snd: Combinator<'x, I, O, Type = (), SType = ()>,
Fst::V: SecureSpecCombinator<Type = <Fst::Type as View>::V>,
Snd::V: SecureSpecCombinator<Type = ()>,
Sourceopen spec fn ex_requires(&self) -> bool
open spec fn ex_requires(&self) -> bool
{ (&self.0, &self.1).ex_requires() }Sourceexec fn serialize<'b>(
&self,
v: Self::SType,
data: &'b mut O,
pos: usize,
) -> Result<usize, SerializeError>
exec fn serialize<'b>( &self, v: Self::SType, data: &'b mut O, pos: usize, ) -> Result<usize, SerializeError>
Sourcetype Type = <Fst as Combinator<'x, I, O>>::Type
type Type = <Fst as Combinator<'x, I, O>>::Type
The result type of parsing
Sourcetype SType = <Fst as Combinator<'x, I, O>>::SType
type SType = <Fst as Combinator<'x, I, O>>::SType
The input type of serialization, often a reference to
Self::Type.
For “structural” formats though (e.g., crate::regular::sequence::Pair and crate::regular::variant::Choice),
this is the tuple/sum of the corresponding Combinator::SType types.Sourceimpl<Fst: SecureSpecCombinator, Snd: SecureSpecCombinator<Type = ()>> SecureSpecCombinator for Terminated<Fst, Snd>
impl<Fst: SecureSpecCombinator, Snd: SecureSpecCombinator<Type = ()>> SecureSpecCombinator for Terminated<Fst, Snd>
Sourceproof fn theorem_serialize_parse_roundtrip(&self, v: Self::Type)
proof fn theorem_serialize_parse_roundtrip(&self, v: Self::Type)
Sourceproof fn theorem_parse_serialize_roundtrip(&self, buf: Seq<u8>)
proof fn theorem_parse_serialize_roundtrip(&self, buf: Seq<u8>)
Sourceopen spec fn is_prefix_secure() -> bool
open spec fn is_prefix_secure() -> bool
{ Fst::is_prefix_secure() && Snd::is_prefix_secure() }Sourceproof fn lemma_prefix_secure(&self, s1: Seq<u8>, s2: Seq<u8>)
proof fn lemma_prefix_secure(&self, s1: Seq<u8>, s2: Seq<u8>)
Sourceproof fn lemma_parse_length(&self, s: Seq<u8>)
proof fn lemma_parse_length(&self, s: Seq<u8>)
Sourceopen spec fn is_productive(&self) -> bool
open spec fn is_productive(&self) -> bool
{ (self.0, self.1).is_productive() }Sourceproof fn lemma_parse_productive(&self, s: Seq<u8>)
proof fn lemma_parse_productive(&self, s: Seq<u8>)
Sourcefn corollary_parse_surjective(&self, v: Self::Type)
fn corollary_parse_surjective(&self, v: Self::Type)
Sourcefn corollary_serialize_injective_contraposition(
&self,
v1: Self::Type,
v2: Self::Type,
)
fn corollary_serialize_injective_contraposition( &self, v1: Self::Type, v2: Self::Type, )
Sourcefn lemma_serialize_productive(&self, v: Self::Type)
fn lemma_serialize_productive(&self, v: Self::Type)
Sourceimpl<Fst: SecureSpecCombinator, Snd: SpecCombinator<Type = ()>> SpecCombinator for Terminated<Fst, Snd>
impl<Fst: SecureSpecCombinator, Snd: SpecCombinator<Type = ()>> SpecCombinator for Terminated<Fst, Snd>
Sourceopen spec fn spec_parse(&self, s: Seq<u8>) -> Option<(int, Self::Type)>
open spec fn spec_parse(&self, s: Seq<u8>) -> Option<(int, Self::Type)>
{
if let Some((n, (v, ()))) = (self.0, self.1).spec_parse(s) {
Some((n, v))
} else {
None
}
}Sourceopen spec fn spec_serialize(&self, v: Self::Type) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::Type) -> Seq<u8>
{ (self.0, self.1).spec_serialize((v, ())) }Sourcetype Type = <Fst as SpecCombinator>::Type
type Type = <Fst as SpecCombinator>::Type
The view of [
Combinator::Result].Sourceimpl<Fst: View, Snd: View> View for Terminated<Fst, Snd>
impl<Fst: View, Snd: View> View for Terminated<Fst, Snd>