pub struct Pair<Fst, Snd, Cont> {
pub fst: Fst,
pub _snd: PhantomData<Snd>,
pub snd: Cont,
}Expand description
Combinator that sequentially applies two combinators, where the second combinator depends on the first one.
Fields
fst: Fstcombinators that contain dependencies
_snd: PhantomData<Snd>phantom data representing the second combinator (it really should be private, but this is a workaround for Verus’s conservative treatment of opaque types, which doesn’t allow field access of opaque types in open spec functions)
snd: Contclosure that captures dependencies and maps them to the dependent combinators
Implementations
Sourceimpl<Fst, Snd, Cont> Pair<Fst, Snd, Cont>where
Fst: View,
Snd: View,
Cont: View<V = GhostFn<<Fst::V as SpecCombinator>::Type, Snd::V>>,
Fst::V: SecureSpecCombinator,
Snd::V: SpecCombinator,
impl<Fst, Snd, Cont> Pair<Fst, Snd, Cont>where
Fst: View,
Snd: View,
Cont: View<V = GhostFn<<Fst::V as SpecCombinator>::Type, Snd::V>>,
Fst::V: SecureSpecCombinator,
Snd::V: SpecCombinator,